# Source: https://github.com/stopwords-iso/stopwords-mr/blob/master/stopwords-mr.txt, https://github.com/6/stopwords-json/edit/master/dist/mr.json
STOP_WORDS = set(
    """
न
अतरी
तो
हें
तें
कां
आणि
जें
जे
मग
ते
मी
जो
परी
गा
हे
ऐसें
आतां
नाहीं
तेथ
हा
तया
असे
म्हणे
काय
कीं
जैसें
तंव
तूं
होय
जैसा
आहे
पैं
तैसा
जरी
म्हणोनि
एक
ऐसा
जी
ना
मज
एथ
या
जेथ
जया
तुज
तेणें
तैं
पां
असो
करी
ऐसी
येणें
जाहला
तेंचि
आघवें
होती
कांहीं
होऊनि
एकें
मातें
ठायीं
ये
सकळ
केलें
जेणें
जाण
जैसी
होये
जेवीं
एऱ्हवीं
मीचि
किरीटी
दिसे
देवा
हो
तरि
कीजे
तैसे
आपण
तिये
कर्म
नोहे
इये
पडे
माझें
तैसी
लागे
नाना
जंव
कीर
अधिक
अनेक
अशी
असलयाचे
असलेल्या
असा
असून
असे
आज
आणि
आता
आपल्या
आला
आली
आले
आहे
आहेत
एक
एका
कमी
करणयात
करून
का
काम
काय
काही
किवा
की
केला
केली
केले
कोटी
गेल्या
घेऊन
जात
झाला
झाली
झाले
झालेल्या
टा
तर
तरी
तसेच
ता
ती
तीन
ते
तो
त्या
त्याचा
त्याची
त्याच्या
त्याना
त्यानी
त्यामुळे
त्री
दिली
दोन
न
पण
पम
परयतन
पाटील
म
मात्र
माहिती
मी
मुबी
म्हणजे
म्हणाले
म्हणून
या
याचा
याची
याच्या
याना
यानी
येणार
येत
येथील
येथे
लाख
व
व्यकत
सर्व
सागित्ले
सुरू
हजार
हा
ही
हे
होणार
होत
होता
होती
होते
""".split()
)
