"""
Example sentences to test spaCy and its language models.

>>> from spacy.lang.gu.examples import sentences
>>> docs = nlp.pipe(sentences)
"""


sentences = [
    "લોકશાહી એ સરકારનું એક એવું તંત્ર છે જ્યાં નાગરિકો મત દ્વારા સત્તાનો ઉપયોગ કરે છે.",
    "તે ગુજરાત રાજ્યના ધરમપુર શહેરમાં આવેલું હતું",
    "કર્ણદેવ પહેલો સોલંકી વંશનો રાજા હતો",
    "તેજપાળને બે પત્ની હતી",
    "ગુજરાતમાં ભારતીય જનતા પક્ષનો ઉદય આ સમયગાળા દરમિયાન થયો",
    "આંદોલનકારીઓએ ચીમનભાઇ પટેલના રાજીનામાની માંગણી કરી.",
    "અહિયાં શું જોડાય છે?",
    "મંદિરનો પૂર્વાભિમુખ ભાગ નાના મંડપ સાથે થોડો લંબચોરસ આકારનો છે.",
]
